/* test */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "odicon.h"
#include "lseparable.h"

/* $B$d$C$?$i$"$+$s(B

quasi L $B$N:G>.2=4p=`$K(B submodular $B$O;H$($J$$(B!!

naive $B$K9T$/$7$+$J$$(B

*/

#define R 0.001

double quasi_transform(double x) {
    double r = floor(x);

    if (floor(r / 2) * 2 == r) return x - (1-R) * r / 2;
    else                       return r - (1-R) * (r-1) / 2 + R * (x - r);
}

void quasi_transform_test(void) {
    double r;

    for (r=-10; r<10; r+=0.1) {
        printf("%10.6f %10.6f\n", r, quasi_transform(r));
    }
}


static void test(int dim, double f(int,int*), int init1[], int init2[],
                 int limit1, int limit2) {
    double ans;
    int x[1000];

    printf("--------------------------\n");
    printf("naive + relax\n");
    ivector_copy(dim, x, init1);
    ans = lgconv_minimize(dim, f, x);

    if (dim > limit1) return;
    printf("--------------------------\n");
    printf("naive + scaling\n");
    ivector_copy(dim, x, init2);
    if (fcmp(ans,lgconv_minimize_scaling(dim, f, x)) != 0) exit(1);

    if (dim > limit2) return;
    printf("--------------------------\n");
    printf("naive + SD\n");
    ivector_copy(dim, x, init2);
    if (fcmp(ans,lgconv_minimize(dim, f, x)) != 0) exit(1);
}

static struct lseparable_st *lseparable_inst;
static double ffunc(int dim, double x[]) { return quasi_transform(lseparable(dim, x, lseparable_inst)); }
/*static double ifunc(int dim, int x[]) { return int_func(dim, ffunc, x); }*/
static double ifunc(int dim, int x[]) { return intint_func(dim, ffunc, x); }

void test_qlconv(const int dim) {
    int i;
    double *fans = fvector_new(dim, 190, -200, 200);
    int    *ians = ivector_new(dim, 190, -200, 200);
    int    *init = ivector_new(dim, 190, -200, 200); /* starting point */

    for (i=0; i<dim; i++) {
        fans[i] = init[i] = irandom(-1000, +1000);
    }
    /*
    {int data[] = {-19, -14, 33, -29};
    for (i=0; i<dim; i++) init[i] = data[i];}
    */

    lseparable_inst = lseparable_make(dim);
#ifdef DEBUG    
    lseparable_output(lseparable_inst);
#endif /* DEBUG */
    lbfgs_call(dim, ffunc, fans);
    for (i=0; i<dim; i++) ians[i] = (int)floor(0.5 + fans[i]);

    test(dim, ifunc, ians, init, 16, 12);
}
